
[Setting name="Enabled" description="Plugin is sending to server"]
bool enabled = true;

[Setting name="Repeat no connection notification" description="When the plugin has no connection, it retries every few minutes. If this setting is enabled it will show a notification each time even if it is still not connected"]
bool repeatConnectionNotification = false;

[Setting name="Opt in to extra tracking" description="By selecting this option, you agree that data on the maps you play and the account with which you play them may be stored, sent to additional servers, and used for data analysis."]
bool data_analysis = false;

[Setting name="Hide connection notifications"]
bool hideNotifications = false;

// last found map if known
string knownmap;

// generated by my server after auth and used to check all messages
string sessionToken = "";

enum Status {
    None,
    NoConnection,
    NotStreamer,
    Streamer
}
Status status = Status::None;

const uint64 retryConnectionWait = 600000;

const uint64 notStreamerWait = 31536000000;

uint64 retryConnectionEnd = 0;

bool shownConnectionNotification = false;

void client_auth() {
    status = Status::None;
    status = preAuth();
    if (status == Status::NotStreamer) {
        return;
    } else if (status == Status::NoConnection) {
        retryConnectionEnd = Time::Now + retryConnectionWait;
        return;
    }
    status = authWithServer(); // full auth with Openplanet servers
    if (status == Status::NotStreamer) {
        // this shouldn't happen unless the plugin is modified to fake the account as a streamer
        print('failed to auth but login is known streamer');
        return;
    } else if (status == Status::NoConnection) {
        retryConnectionEnd = Time::Now + retryConnectionWait;
        return;
    }
}

void client_main() {
    while (true) {
        if (enabled) {
            string currentmap = getMapUid();
            if (currentmap != knownmap) {
                knownmap = currentmap;
                if (currentmap != "" && currentmap != "Unassigned") {
                    // trace('sending info to server');
                    Json::Value@ info = getInfo();
                    if (!(info is null)) {
                        if (!sendToServer(info)) {
                            // either no connection or invalid login but will fully reset
                            // so that when server is back it should return to normal
                            status = Status::NoConnection;
                            return;
                        }
                    }
                }
            }
        }
        sleep(100);
    }
}

void Main() {
    yield();
    // if connection fails, will retry auth
    while (true) {
        // if no connection, wait until end time (usually set to 10 mins but button can move it to now)
        if ((status == Status::NotStreamer || status == Status::NoConnection) && Time::Now < retryConnectionEnd) {
            sleep(100);
            continue;
        }
        // if no connection (and time ended) or no status (e.g. first start), try auth
        if (status == Status::NoConnection || status == Status::None) {
            client_auth();
        }
        // if not streamer after auth, set no connection time to a long time in the future
        // (this way, to have a button to re-check we can just set end to now)
        if (status == Status::NotStreamer) {
            retryConnectionEnd = Time::Now + notStreamerWait;
        }
        // is streamer so start watching current map
        if (status == Status::Streamer) {
            shownConnectionNotification = false;
            client_main();
        }
    }
}

void RenderMenuMain() {
    if (status == Status::NotStreamer) {
        if (UI::MenuItem(Meta::ExecutingPlugin().Name + ": \\$faa not a streamer", "", false)) {
            // clicking this retries login each time (in case this is useful to people), after 10 seconds
            if (retryConnectionEnd - Time::Now < notStreamerWait - 10000) {
                status = Status::NoConnection;
                retryConnectionEnd = Time::Now;
            } else {
                UI::ShowNotification(Meta::ExecutingPlugin().Name, "You must wait 10 seconds to try connecting again", vec4(0.5f, 0.f, 0.f, 0.5f));
            }
        }
    } else if (status == Status::NoConnection) {
        if (UI::MenuItem(Meta::ExecutingPlugin().Name + ": \\$faa connection error", "", false)) {
            if (retryConnectionEnd - Time::Now < retryConnectionWait - 10000) {
                retryConnectionEnd = Time::Now;
            } else {
                UI::ShowNotification(Meta::ExecutingPlugin().Name, "You must wait 10 seconds to try connecting again", vec4(0.5f, 0.f, 0.f, 0.5f));
            }
        }
    } else if (status == Status::Streamer) {
        if (UI::MenuItem(Meta::ExecutingPlugin().Name + (enabled ? ": \\$afa enabled" : ": \\$faa disabled"), "", enabled)) {
            enabled = !enabled;
            Meta::SaveSettings();
        }
    } else if (status == Status::None) {
        UI::MenuItem(Meta::ExecutingPlugin().Name + ": \\$z Connecting ...", "", false);
    }
}